/**
 *
 * Copyright (C) 2013-2014, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */
#ifndef __MLNX_BSP_H_
#define __MLNX_BSP_H_

#ifndef __KERNEL__
#define __user 
#endif

#define CPLD_LED_NUM           9
#define ASIC_FAN_NUM           6
#define ASIC_FAN_TACH_NUM      6
#define ASIC_TEMP_SENSORS_NUM  5
#define BSP_ENTRY_NAME        32

typedef enum led_index {
        led_status,
        led_ps,
        led_ps1,
        led_ps2,
        led_fan,
        led_fan1,
        led_fan2,
        led_fan3,
        led_fan4,
} led_index_t;

typedef enum fan_index {
        asic_fan1_fan,
        asic_fan2_fan,
        asic_fan3_fan,
        asic_fan4_fan,
        asic_ps1_fan,
        asic_ps2_fan,
} fan_index_t;

typedef enum temp_index {
        asic_temp1,
        asic_temp2,
        asic_temp1_port,
        asic_temp2_port,
        asic_temp3_port,
} temp_index_t;

struct mlnx_bsp_entry {
        __u8 index;           // Entry index
        char name[BSP_ENTRY_NAME];  // Entry name      
};

struct mlnx_cpld_led_config {
	struct mlnx_bsp_entry entry; // Entry id
        __u8 offset;          // Led offset within CPLD address space 
        __u8 access_mask;     // Led access mask       
};

struct mlnx_cpld_leds_config_ioctl_params {
	__u16 device_type;         // Device attached to bus I2C_BUS_TYPE or to LPC_BUS_TYPE 
	int dev_id;                // Device id
	union hw_dev hwdev;
        __u8 num_leds;
        struct mlnx_cpld_led_config cpld_leds[CPLD_LED_NUM];
};

struct mlnx_asic_fan_config {
	struct mlnx_bsp_entry entry;      // Entry id
        __u8 num_tachos;
	__u8 tacho_id[ASIC_FAN_TACH_NUM]; // Fan tachometer index
	__u8 pwm_id;                      // PWM tachometer index
	__u8 fan_rot_percent;             // Fan speed rotation
	__u16 speed[ASIC_FAN_TACH_NUM];   // Fan speed (Round Per Minute) calculated based on the time measurement between n
                                          // fan pulses. Note that in order for the RPM to be correct, the n value should
                                          // correspond to the number of Tach pulses per rotation measured by the tachometer. 
	__u8 pwm_duty_cycle;              // Controls the duty cycle of the PWM. Value range from 0..255
	__u8 asic_dev_id;                 // ASIC device Id
};

struct mlnx_asic_fan_config_ioctl_params {
        __u8 num_fans;
        struct mlnx_asic_fan_config asic_fans[ASIC_FAN_NUM];
};

struct mlnx_asic_temp_sensor_config {
	struct mlnx_bsp_entry entry; // Entry id
        __u8 sensor_index;           // Sensors index to access
        __u32 temperature;           // Temperature reading from the sensor. Reading in 0.125 Celsius degrees
        __u8 mte;                    // Enables measuring the max temperature on a sensor
        __u8 mtr;                    // Clears the value of the max temperature register
        __u32 max_temperature;       // The highest measured temperature from the sensor
        __u8 tee;                    // Temperature Event Enable
        __u32 temperature_threshold; // Generate event if sensor temperature measurement is above the threshold and events enabled
	__u8 asic_dev_id;            // ASIC device Id
};

struct mlnx_asic_temp_sensor_config_ioctl_params {
        __u8 num_sensors;
        struct mlnx_asic_temp_sensor_config asic_sensors[ASIC_TEMP_SENSORS_NUM];
};

extern int (*mlnx_cpld_led_config_symbol_hook)(struct mlnx_cpld_leds_config_ioctl_params *p_cpld_led_config);
extern int (*mlnx_asic_fan_config_symbol_hook)(struct mlnx_asic_fan_config_ioctl_params *p_asic_fan_config);
extern int (*mlnx_asic_temp_sensor_config_symbol_hook)(struct mlnx_asic_temp_sensor_config_ioctl_params *p_asic_temp_sensor);

#endif
