/**
 *
 * Copyright (C) 2013-2014, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */
#ifndef __MLNX_IRQ_H_
#define __MLNX_IRQ_H_

#ifndef __KERNEL__
#define __user 
#endif

#include "mlnx_hwdev.h"

#define MAX_MODULE_NUM             54
#define MLNX_CPLD_IRQ_SUBSRIBE   0x1C
#define MLNX_CPLD_IRQ_UNSUBSRIBE 0x1D
#define MLNX_CPLD_IRQ_SET        0x1E
#define MLNX_CPLD_IRQ_GET_DATA   0x1F

#define NETLINK_SYS_IRQ_SOCK         NETLINK_USERSOCK
#define SYS_IRQ_NL_CONTROL           (NLMSG_MIN_TYPE)
#define SYS_IRQ_NL_REGISTER          (NLMSG_MIN_TYPE + 1)
#define SYS_IRQ_NL_UNREGISTER        (NLMSG_MIN_TYPE + 2)
#define SYS_IRQ_NL_INTERRUPT         (NLMSG_MIN_TYPE + 3)
#define SYS_IRQ_NL_HANDLER_ERR       (NLMSG_MIN_TYPE + 4)
#define SYS_IRQ_NL_RCVRD_ERR         (NLMSG_MIN_TYPE + 5)
#define SYS_IRQ_NL_FALSE_INT_ALARM   (NLMSG_MIN_TYPE + 6)
#define DRV_MAX_MSGLEN         512

#define CPLD_IRQ_HANDLER_ENABLE  0
#define CPLD_IRQ_HANDLER_DISABLE 1

typedef enum irq_entry_type {
	_int_ps1,            // Interrupt PSs 1'
	_int_ps2,            // Interrupt PSs 2'
	_mezz_pg,            // Mezzanine Power Good
	_aggr,               // Interrupt Aggregation
	_cpld2,              // CPLD 2 Aggregation
	_cpld3,              // CPLD 3 Aggregation
	_cpld4,              // CPLD 4 Aggregation
	_board1,             // Interrupt Board 1
	_board2,             // Interrupt Board 2
	_board3,             // Interrupt Board 3
	_asic,               // ASIC
	_ha_int,             // HA Int
	_prsnt_ps1,          // Present PSs' 1
	_prsnt_ps2,          // Present PSs' 2
	_int_ac_good_ps1,    // Interrupt PSs' AC Fail 1
	_int_ac_good_ps2,    // Interrupt PSs' AC Fail 2
	_int_dc_good_ps1,    // Interrupt PSs' DC Fail 1
	_int_dc_good_ps2,    // Interrupt PSs' DC Fail 2
	_int_temp_alarm_ps1, // Interrupt PSs' Temperatures'Alarm 1
	_int_temp_alarm_ps2, // Interrupt PSs' Temperatures'Alarm 2
	_prsnt_leaf_1_8,     // Present Leafs'
	_prsnt_leaf_9_16,    // Present Leafs'
	_prsnt_leaf_17_24,   // Present Leafs'
	_prsnt_leaf_25_32,   // Present Leafs'
	_prsnt_leaf_33_36,   // Present Leafs'
	_prsnt_spine_1_8,    // Present Spines'
	_prsnt_spine_9_16,   // Present Spines'
	_prsnt_spine_17_18,  // Present Spines'
	_prsnt_fan1,         // Present Fans' 1
	_prsnt_fan2,         // Present Fans' 2
	_prsnt_fan3,         // Present Fans' 3
	_int_fan1,           // Interrupt Fans' 1
	_int_fan2,           // Interrupt Fans' 2
	_int_fan3,           // Interrupt Fans' 3
	_int_leaf_1_8,       // Interrupt Leafs'
	_int_leaf_9_16,      // Interrupt Leafs'
	_int_leaf_17_24,     // Interrupt Leafs'
	_int_leaf_25_32,     // Interrupt Leafs'
	_int_leaf_33_36,     // Interrupt Leafs'
	_int_spine_1_8,      // Interrupt Spines'
	_int_spine_9_16,     // Interrupt Spines'
	_int_spine_17_18,    // Interrupt Spines'
	_ha_int_far_end,     // HA Int far end
	_wd,                 // Watchdog
} irq_entry_type_e;

#define MAX_MGMT_IRQS (_wd - _int_ps1 + 1)
#define MAX_MOD_IRQS  MAX_MGMT_IRQS
#define MAX_TOR_IRQS  MAX_MGMT_IRQS

/* Level defines in which order interrup is to be handled  
*/
enum irq_level_e {
        INT_NONE        = 0x0000,
        INT_LEVEL_1     = 0x0001,
        INT_LEVEL_2     = 0x0002,
        INT_LEVEL_3     = 0x0004,
        INT_LEVEL_1_2   = 0x0008,
        INT_LEVEL_2_3   = 0x0010,
        INT_LEVEL_1_2_3 = 0x0020,
        INT_LEVEL_HA    = 0x0040,
        INT_LEVEL_4     = 0x0080,
        INT_LEVEL_5     = 0x0100,
        INT_LEVEL_4_5   = 0x0200,
};
#define INT_LEVEL_MASK    0x7f
#define INT_LEVEL_DEFAULT INT_LEVEL_MASK

struct sys_irq_nl_msg {
	unsigned int data_len;
	unsigned char data[];
};

typedef struct sys_irq_reg {
	unsigned int pid;
	unsigned int int_line;
} sys_irq_reg_t;

struct irq_digest {
	unsigned char int_vector;                       // Module's interrupt vector
	unsigned char int_status;                       // Module's interrupt status
};

struct mod_digest {
        int usr_handle;                                 // Module's user handle
        unsigned char change_cntr;                      // Number of entries when event change is detected
        struct irq_digest int_digest[MAX_MGMT_IRQS];    // Module's interrupt digest
};
 
struct int_digest_blk_t {
        unsigned char mod_num;                          // Number of modules, where interrupts are collected
        int all_change_cntr;                            // Summary of all entries when event change is detected
        struct mod_digest digest_table[MAX_MODULE_NUM]; // Table of collected digest           
};

struct mod_digest_send {
        unsigned char irq_entry_id;                     // Entry Id accroding to irq_entry_type
        struct irq_digest int_digest;                   // Entry mask - bit 1 means interrupt is occured 
};

struct int_module_digest_send_t {
        int usr_handle;                                 // Module's user handle
        unsigned char num_entry;                        // Number of entries with interrupt indication
        struct mod_digest_send digest_tab[];            // Module's interrupt mask table
};

struct int_blk_digest_send_t {
        unsigned char mod_num;                          // Number of modules, where interrupts are collected
        struct int_module_digest_send_t all_digest[];   // Per module digest entry
};

#endif /*__MLNX_IRQ_H_*/
